# -*- coding: utf-8 -*-
from Acquisition import aq_base
from Acquisition import aq_get
from plone.app.contentlisting.contentlisting import BaseContentListingObject
from plone.app.contentlisting.interfaces import IContentListingObject
from plone.registry.interfaces import IRegistry
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.interface import implementer


missing = object()


@implementer(IContentListingObject)
class CatalogContentListingObject(BaseContentListingObject):
    """A Catalog-results based content object representation.

    Whenever sequences of catalog brains are turned into contentlistings,
    This is the type of objects they are adapted to.
    """

    def __init__(self, brain):
        self._brain = brain
        self._cached_realobject = None
        self.request = aq_get(brain, 'REQUEST')

    def __repr__(self):
        return '<plone.app.contentlisting.catalog.'\
            'CatalogContentListingObject instance at {0}>'.format(
                self.getPath()
            )

    __str__ = __repr__

    def __getattr__(self, name):
        """We'll override getattr so that we can defer name lookups to the real
        underlying objects without knowing the names of all attributes.
        """

        if name.startswith('_'):
            raise AttributeError(name)
        brain_name = getattr(aq_base(self._brain), name, missing)
        if brain_name is not missing:
            return brain_name
        object_name = getattr(aq_base(self.getObject()), name, missing)
        if object_name is not missing:
            return object_name
        raise AttributeError(name)

    def getDataOrigin(self):
        # The origin of the data for the object.
        # Sometimes we just need to know if we are looking at a brain or
        # the real object.
        if self._cached_realobject is not None:
            return self._cached_realobject
        else:
            return self._brain

    def getObject(self):
        # Get the real, underlying object.

        # This is performance intensive compared to just getting the
        # catalog brain, so we don't do it until we need to.  We may
        # even have to log this to notify the developer that this might
        # be an inefficient operation.
        if self._cached_realobject is None:
            self._cached_realobject = self._brain.getObject()
        return self._cached_realobject

    # a base set of elements that are needed but not defined in dublin core
    def getId(self):
        return self._brain.getId

    def getPath(self):
        return self._brain.getPath()

    def getURL(self, relative=False):
        return self._brain.getURL(relative=relative)

    def uuid(self):
        # content objects might have UID and might not.
        brain_uid = getattr(aq_base(self._brain), 'UID', None)
        if brain_uid is not None:
            return self._brain.UID
        uuid = IUUID(self.getObject(), None)
        if uuid is not None:
            return uuid
        return self.getPath()

    def getSize(self):
        return self._brain.getObjSize

    def review_state(self):
        return self._brain.review_state

    # All the dublin core elements. Most of them should be in the
    # brain for easy access
    def Title(self):
        return self._brain.Title

    def Description(self):
        return self._brain.Description

    def CroppedDescription(self):
        registry = queryUtility(IRegistry)
        length = registry.get('plone.search_results_description_length')
        plone_view = getMultiAdapter((self, self.request), name='plone')
        return plone_view.cropText(self.Description(), length)

    def Type(self):
        return self._brain.Type

    def PortalType(self):
        return self._brain.portal_type

    def listCreators(self):
        return self._brain.listCreators

    def getUserData(self, username):
        _usercache = self.request.get('usercache', None)
        if _usercache is None:
            self.request.set('usercache', {})
            _usercache = {}
        userdata = _usercache.get(username, None)
        if userdata is None:
            membershiptool = getToolByName(self._brain, 'portal_membership')
            userdata = membershiptool.getMemberInfo(self._brain.Creator)
            if not userdata:
                userdata = {
                    'username': username,
                    'description': '',
                    'language': '',
                    # TODO
                    # string:${navigation_root_url}/author/${item_creator}
                    'home_page': '/HOMEPAGEURL',
                    'location': '',
                    'fullname': username
                }
            self.request.usercache[username] = userdata
        return userdata

    def Creator(self):
        username = self._brain.Creator
        return username

    def Author(self):
        return self.getUserData(self.Creator())

    def Subject(self):
        return self._brain.Subject

    def Publisher(self):
        raise NotImplementedError

    def listContributors(self):
        raise NotImplementedError

    def Contributors(self):
        return self.listContributors()

    def Date(self, zone=None):
        return self._brain.Date

    def CreationDate(self, zone=None):
        return self._brain.CreationDate

    def EffectiveDate(self, zone=None):
        return self._brain.EffectiveDate

    def ExpirationDate(self, zone=None):
        return self._brain.ExpirationDate

    def ModificationDate(self, zone=None):
        return self._brain.ModificationDate

    def Format(self):
        raise NotImplementedError

    def Identifier(self):
        return self.getURL()

    def Language(self):
        # The language of the content.
        brain_language = getattr(aq_base(self._brain), 'Language', None)
        if brain_language is not None:
            return self._brain.Language
        else:
            return self.getObject().Language()

    def Rights(self):
        raise NotImplementedError



# -*- coding: utf-8 -*-
from Products.CMFCore.interfaces import IDublinCore
from zope.interface.common.sequence import IReadSequence


class IContentListing(IReadSequence):
    """Sequence of IContentListingObjects."""


class IContentListingObject(IDublinCore):
    """Unified representation of content objects in listings."""

    def getDataOrigin():
        """The origin of the data for the object."""

    def getObject():
        """get the real object (may be expensive)."""

    def getId():
        """get the object id in its container."""

    def getPath():
        """Path to the object, relative to the portal root."""

    def getURL(relative=False):
        """Full url to the object, including the portal root."""

    def uuid():
        """Unique content identifier."""

    def getSize():
        """size in bytes."""

    def review_state():
        """Workflow review state."""

    def Title():
        """Title."""

    def Description():
        """Description."""

    def CroppedDescription():
        """A cropped description."""

    def Type():
        """Type title."""

    def PortalType():
        """Content type id of the object."""

    def listCreators():
        """List creators of the object."""

    def getUserData(username):
        """Get some data of a given user."""

    def Creator():
        """Creator of the object."""

    def Author():
        """Author of the object."""

    def Subject():
        """Subject(s) of the object."""

    def Date():
        """Date of the object."""

    def CreationDate():
        """Creation date of the object."""

    def EffectiveDate():
        """Date, when content will be shown in listings."""

    def ExpirationDate():
        """Date, when content will be removed from listings."""

    def ModificationDate():
        """Date, when object was last modified."""

    def Language():
        """Language of the object."""

    def ContentTypeClass():
        """The contenttype suitable as a css class name, matching Plone
        conventions.
        """



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from Acquisition import aq_get
from plone.app.contentlisting.contentlisting import BaseContentListingObject
from plone.app.contentlisting.interfaces import IContentListingObject
from plone.rfc822.interfaces import IPrimaryFieldInfo
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from zope.interface import implementer


@implementer(IContentListingObject)
class RealContentListingObject(BaseContentListingObject):
    """A content object representation wrapping a real content object.
    """

    def __init__(self, obj):
        self._realobject = obj
        self.request = aq_get(obj, 'REQUEST')

    def __repr__(self):
        return '<plone.app.contentlisting.realobject.'\
            'RealContentListingObject instance at {0}>'.format(
                self.getPath()
            )

    __str__ = __repr__

    def __getattr__(self, name):
        """We'll override getattr so that we can defer name lookups to
        the real underlying objects without knowing the names of all
        attributes.
        """
        if name.startswith('_'):
            raise AttributeError(name)
        obj = self.getObject()
        obj_name = getattr(aq_base(obj), name, None)
        if obj_name is not None:
            return obj_name
        else:
            raise AttributeError(name)

    def getObject(self):
        return self._realobject

    def getDataOrigin(self):
        # The origin of the data for the object.
        # Sometimes we just need to know if we are looking at a brain or
        # the real object.
        return self.getObject()

    # a base set of elements that are needed but not defined in dublin core
    def getPath(self):
        return '/'.join(self.getObject().getPhysicalPath())

    def getURL(self):
        return self.getObject().absolute_url()

    def uuid(self):
        # content objects might have UID and might not. Same thing for
        # their brain.
        uuid = IUUID(self.getObject(), None)
        if uuid is not None:
            return uuid
        return self.getPath()

    def getSize(self):
        # return size of the primary field content.
        # this works the same way as the indexer works
        obj = self.getObject()
        try:
            primary_field_info = IPrimaryFieldInfo(obj, None)
        except TypeError:
            # no primary field available, dexterity raises a TypeError
            # with the slightly missleading message 'could not adapt'.
            return 0
        if primary_field_info is None or not primary_field_info.value:
            return 0
        return obj.getObjSize(
            None,
            getattr(primary_field_info.value, 'size', 0)
        )

    def review_state(self):
        obj = self.getObject()
        wftool = getToolByName(obj, 'portal_workflow')
        return wftool.getInfoFor(obj, 'review_state')

    def Type(self):
        # Dublin Core element - Object type.
        obj = self.getObject()
        typestool = getToolByName(obj, 'portal_types')
        ti = typestool.getTypeInfo(obj)
        if ti is not None:
            return ti.Title()
        return obj.meta_type

# Needed: A method Type() that returns the same as is cataloged as Type.
# Currently Type() returns different values depending on the data source being
# a brain or a real object. Probably needed. Support for all the attributes
# from the indexablemetadata wrappers.

    def PortalType(self):
        obj = self.getObject()
        return obj.portal_type



# -*- coding: utf-8 -*-
from plone.app.contentlisting.tests.base import CONTENTLISTING_FUNCTIONAL_TESTING  # noqa
from plone.testing import layered

import doctest
import unittest


def test_suite():
    return unittest.TestSuite([
        layered(doctest.DocFileSuite(
            'tests/integration.rst',
            package='plone.app.contentlisting',
            optionflags=doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS),
            layer=CONTENTLISTING_FUNCTIONAL_TESTING,
        ),
    ])
